=== modified file 'Makefile'
--- Makefile	2016-09-30 03:15:55 +0000
+++ Makefile	2016-10-12 15:26:17 +0000
@@ -273,7 +273,7 @@
 	@$(error Use `$(MAKE) test+coverage` to generate coverage data, \
 	    or invoke a test script using the `--with-coverage` flag)
 
-lint: lint-py lint-py-imports lint-js lint-doc lint-rst
+lint: lint-py lint-py-complexity lint-py-imports lint-js lint-doc lint-rst
 
 pocketlint = $(call available,pocketlint,python-pocket-lint)
 
@@ -283,19 +283,27 @@
 	@find $(sources) -type f \
 	    -print0 | xargs -r0 $(pocketlint) --max-length=120
 
-# Python lint checks are time-intensive, so we run them in parallel.  It may
-# make things matters worse if the files need to be read from disk, though, so
-# this may need more tuning.
-# The -n50 -P4 setting roughly doubled speed on a high-end system with SSD and
-# all the files in cache.
-lint-py: sources = $(wildcard *.py contrib/*.py) src templates twisted utilities etc
+# Python lint checks are time-intensive, but flake8 now knows how to run
+# parallel jobs, and does so by default.
+lint-py: sources = \
+    setup.py $(wildcard contrib/*.py) src templates twisted utilities etc
 lint-py: bin/flake8
 	@find $(sources) -name '*.py' \
+	  ! -path '*/migrations/*' ! -path '*/south_migrations/*' -print0 \
+	    | xargs -r0 bin/flake8 --ignore=E123,E402,E731 --isolated
+
+# Ignore tests when checking complexity. The maximum complexity ought to
+# be close to 10 but MAAS has many functions that are over that so we
+# start with a much higher number. Over time we can ratchet it down.
+lint-py-complexity: maximum=26
+lint-py-complexity: sources = \
+    setup.py $(wildcard contrib/*.py) src templates twisted utilities etc
+lint-py-complexity: bin/flake8
+	@find $(sources) -name '*.py' \
 	  ! -path '*/migrations/*' ! -path '*/south_migrations/*' \
-	  ! -path 'src/provisioningserver/twisted/*' ! -path 'ez_setup.py' \
-	  ! -path 'src/maasserver/data/*' -print0 \
-	    | xargs -r0 -n50 -P4 bin/flake8 --ignore=E123,E402,E731 \
-	    --config=/dev/null
+	  ! -path '*/tests/*' ! -path '*/testing/*' ! -name 'testing.py' \
+	  -print0 | xargs -r0 bin/flake8 --ignore=E123,E402,E731 \
+	              --isolated --max-complexity=$(maximum)
 
 # Statically check imports against policy.
 lint-py-imports:
@@ -449,6 +457,7 @@
   lint-doc
   lint-js
   lint-py
+  lint-py-complexity
   lint-py-imports
   lint-rst
   lxd

=== modified file 'buildout.cfg'
--- buildout.cfg	2016-07-13 10:44:36 +0000
+++ buildout.cfg	2016-10-12 15:26:17 +0000
@@ -303,7 +303,7 @@
 eggs =
   flake8
 entry-points =
-  flake8=flake8.run:main
+  flake8=flake8.main.cli:main
 initialization =
   ${common:path-munge}
   ${common:warnings}

=== modified file 'contrib/preseeds_v2/curtin_userdata'
--- contrib/preseeds_v2/curtin_userdata	2016-10-05 20:08:01 +0000
+++ contrib/preseeds_v2/curtin_userdata	2016-10-12 15:26:17 +0000
@@ -14,7 +14,7 @@
   driver_04_load: ["sh", "-c", "depmod && modprobe {{driver['module']}}"]
 {{endif}}
 late_commands:
-  maas: [wget, '--no-proxy', {{node_disable_pxe_url|escape.shell}}, '--post-data', {{node_disable_pxe_data|escape.shell}}, '-O', '/dev/null']
+  maas: [wget, '--no-proxy', '{{node_disable_pxe_url|escape.shell}}', '--post-data', '{{node_disable_pxe_data|escape.shell}}', '-O', '/dev/null']
 {{if third_party_drivers and driver}}
   driver_00_key_get: curtin in-target -- sh -c "/bin/echo -en '{{key_string}}' > /tmp/maas-{{driver['package']}}.gpg"
   driver_02_key_add: ["curtin", "in-target", "--", "apt-key", "add", "/tmp/maas-{{driver['package']}}.gpg"]

=== modified file 'src/maasserver/api/account.py'
--- src/maasserver/api/account.py	2016-08-09 21:59:28 +0000
+++ src/maasserver/api/account.py	2016-10-12 15:26:17 +0000
@@ -5,7 +5,7 @@
 
 __all__ = [
     'AccountHandler',
-    ]
+]
 
 import http.client
 import json
@@ -15,10 +15,27 @@
     operation,
     OperationsHandler,
 )
-from maasserver.api.utils import get_mandatory_param
+from maasserver.api.utils import (
+    get_mandatory_param,
+    get_optional_param,
+)
 from piston3.utils import rc
 
 
+def _format_tokens(tokens):
+    """Converts the given tokens into a list of dictionaries to represent them.
+
+    :param tokens: The result of `profile.get_authorisation_tokens()`.
+    """
+    return [
+        {
+            "name": token.consumer.name,
+            "token": ":".join([token.consumer.key, token.key, token.secret])
+        }
+        for token in tokens
+    ]
+
+
 class AccountHandler(OperationsHandler):
     """Manage the current logged-in user."""
     api_doc_section_name = "Logged-in user"
@@ -28,19 +45,22 @@
     def create_authorisation_token(self, request):
         """Create an authorisation OAuth token and OAuth consumer.
 
-        :return: a json dict with three keys: 'token_key',
-            'token_secret' and 'consumer_key' (e.g.
+        :param name: Optional name of the token that will be generated.
+        :type name: unicode
+        :return: a json dict with four keys: 'token_key',
+            'token_secret', 'consumer_key'  and 'name'(e.g.
             {token_key: 's65244576fgqs', token_secret: 'qsdfdhv34',
-            consumer_key: '68543fhj854fg'}).
+            consumer_key: '68543fhj854fg', name: 'MAAS consumer'}).
         :rtype: string (json)
 
         """
         profile = request.user.userprofile
-        consumer, token = profile.create_authorisation_token()
+        consumer_name = get_optional_param(request.data, 'name')
+        consumer, token = profile.create_authorisation_token(consumer_name)
         auth_info = {
             'token_key': token.key, 'token_secret': token.secret,
-            'consumer_key': consumer.key,
-            }
+            'consumer_key': consumer.key, 'name': consumer.name
+        }
         return HttpResponse(
             json.dumps(auth_info),
             content_type='application/json; charset=utf-8',
@@ -58,6 +78,39 @@
         profile.delete_authorisation_token(token_key)
         return rc.DELETED
 
+    @operation(idempotent=False)
+    def update_token_name(self, request):
+        """Modify the consumer name of an authorisation OAuth token.
+
+        :param token: Can be the whole token or only the token key.
+        :type token: unicode
+        :param name: New name of the token.
+        :type name: unicode
+        """
+        profile = request.user.userprofile
+        token = get_mandatory_param(request.data, 'token')
+        token_fields = token.split(":")
+        if len(token_fields) == 3:
+            token_key = token_fields[1]
+        else:
+            token_key = token
+        consumer_name = get_mandatory_param(request.data, 'name')
+        profile.modify_consumer_name(token_key, consumer_name)
+        return rc.ACCEPTED
+
+    @operation(idempotent=True)
+    def list_authorisation_tokens(self, request):
+        """List authorisation tokens available to the currently logged-in user.
+
+        :return: list of dictionaries representing each key's name and token.
+        """
+        profile = request.user.userprofile
+        tokens = _format_tokens(profile.get_authorisation_tokens())
+        return HttpResponse(
+            json.dumps(tokens, indent=4),
+            content_type='application/json; charset=utf-8',
+            status=int(http.client.OK))
+
     @classmethod
     def resource_uri(cls, *args, **kwargs):
         return ('account_handler', [])

=== modified file 'src/maasserver/api/tests/test_api.py'
--- src/maasserver/api/tests/test_api.py	2016-09-23 19:21:31 +0000
+++ src/maasserver/api/tests/test_api.py	2016-10-12 15:26:17 +0000
@@ -181,7 +181,8 @@
 
     def test_create_authorisation_token(self):
         # The api operation create_authorisation_token returns a json dict
-        # with the consumer_key, the token_key and the token_secret in it.
+        # with the consumer_key, the token_key, the token_secret and the
+        # consumer_name in it.
         response = self.client.post(
             reverse('account_handler'), {'op': 'create_authorisation_token'})
         self.assertEqual(http.client.OK, response.status_code)
@@ -189,11 +190,27 @@
             'application/json; charset=utf-8', response["content-type"])
         parsed_result = json_load_bytes(response.content)
         self.assertEqual(
-            ['consumer_key', 'token_key', 'token_secret'],
+            ['consumer_key', 'name', 'token_key', 'token_secret'],
             sorted(parsed_result))
         self.assertIsInstance(parsed_result['consumer_key'], str)
         self.assertIsInstance(parsed_result['token_key'], str)
         self.assertIsInstance(parsed_result['token_secret'], str)
+        self.assertIsInstance(parsed_result['name'], str)
+
+    def test_create_authorisation_token_with_token_name(self):
+        # The api operation create_authorisation_token can also accept
+        # a new name for the generated token.
+        token_name = 'Test_Token'
+        response = self.client.post(
+            reverse('account_handler'), {
+                'op': 'create_authorisation_token',
+                'name': token_name
+            })
+        self.assertEqual(http.client.OK, response.status_code)
+        self.assertEqual(
+            'application/json; charset=utf-8', response["content-type"])
+        parsed_result = json_load_bytes(response.content)
+        self.assertEqual(parsed_result['name'], token_name)
 
     def test_delete_authorisation_token_not_found(self):
         # If the provided token_key does not exist (for the currently
@@ -209,9 +226,131 @@
         # delete_authorisation_token. It it is not present in the request's
         # parameters, the api returns a 'Bad Request' (400) error.
         response = self.client.post(
-            reverse('account_handler'), {'op': 'delete_authorisation_token'})
-
-        self.assertEqual(http.client.BAD_REQUEST, response.status_code)
+            reverse('account_handler'), {
+                'op': 'delete_authorisation_token'
+            })
+
+        self.assertEqual(http.client.BAD_REQUEST, response.status_code)
+
+    def test_update_authorisation_token(self):
+        token_name_orig = 'Test_Token'
+        token_name_updated = 'Test_Token update'
+        response_creation = self.client.post(
+            reverse('account_handler'), {
+                'op': 'create_authorisation_token',
+                'name': token_name_orig
+            })
+        parsed_creation_response = json_load_bytes(response_creation.content)
+        created_token = ":".join([
+            parsed_creation_response['consumer_key'],
+            parsed_creation_response['token_key'],
+            parsed_creation_response['token_secret']
+        ])
+        self.client.post(
+            reverse('account_handler'), {
+                'op': 'update_token_name', 'token': created_token,
+                'name': token_name_updated
+            })
+        response_list = self.client.get(
+            reverse('account_handler'), {
+                'op': 'list_authorisation_tokens'
+            })
+        parsed_list_response = json_load_bytes(response_list.content)
+        for token in parsed_list_response:
+            if token['token'] == created_token:
+                self.assertEqual(token['name'], token_name_updated)
+
+    def test_update_authorisation_token_with_token_key(self):
+        # We use only "token_key" portion of the authorisation token
+        # to update the token name.
+        token_name_orig = 'Test_Token'
+        token_name_updated = 'Test_Token update'
+        response_creation = self.client.post(
+            reverse('account_handler'), {
+                'op': 'create_authorisation_token',
+                'name': token_name_orig
+            })
+        parsed_creation_response = json_load_bytes(response_creation.content)
+        created_token = ":".join([
+            parsed_creation_response['consumer_key'],
+            parsed_creation_response['token_key'],
+            parsed_creation_response['token_secret']
+        ])
+        self.client.post(
+            reverse('account_handler'), {
+                'op': 'update_token_name',
+                'token': parsed_creation_response['token_key'],
+                'name': token_name_updated
+            })
+        response_list = self.client.get(
+            reverse('account_handler'), {
+                'op': 'list_authorisation_tokens'
+            })
+        parsed_list_response = json_load_bytes(response_list.content)
+        for token in parsed_list_response:
+            if token['token'] == created_token:
+                self.assertEqual(token['name'], token_name_updated)
+
+    def test_update_authorisation_token_name_not_found(self):
+        # If the provided token_key does not exist (for the currently
+        # logged-in user), the api returns a 'Not Found' (404) error.
+        response = self.client.post(
+            reverse('account_handler'), {
+                'op': 'update_token_name', 'token': 'no-such-token',
+                'name': 'test_name'
+            })
+
+        self.assertEqual(http.client.NOT_FOUND, response.status_code)
+
+    def test_update_authorisation_token_name_bad_request_no_token(self):
+        # `token` and `name` are mandatory parameters when calling
+        # update_token_name. If it is not present in the request's
+        # parameters, the api returns a 'Bad Request' (400) error.
+        response = self.client.post(
+            reverse('account_handler'), {
+                'op': 'update_token_name'
+            })
+
+        self.assertEqual(http.client.BAD_REQUEST, response.status_code)
+
+    def test_list_tokens(self):
+        token1_name = "Test Token 1"
+        response_creation = self.client.post(
+            reverse('account_handler'), {
+                'op': 'create_authorisation_token',
+                'name': token1_name
+            })
+        parsed_creation_response = json_load_bytes(response_creation.content)
+        response = self.client.get(
+            reverse('account_handler'), {
+                'op': 'list_authorisation_tokens'
+            })
+        parsed_list_response = json_load_bytes(response.content)
+        self.assertEqual(len(json_load_bytes(response.content)), 2)
+        for token in parsed_list_response:
+            if token['name'] == token1_name:
+                token_fields = token['token'].split(":")
+                self.assertEqual(
+                    token_fields[0],
+                    parsed_creation_response['consumer_key'])
+                self.assertEqual(
+                    token_fields[1],
+                    parsed_creation_response['token_key'])
+                self.assertEqual(
+                    token_fields[2],
+                    parsed_creation_response['token_secret'])
+
+    def test_list_tokens_format(self):
+        self.client.post(
+            reverse('account_handler'), {'op': 'create_authorisation_token'})
+        response = self.client.get(
+            reverse('account_handler'), {'op': 'list_authorisation_tokens'})
+        parsed_list_response = json_load_bytes(response.content)
+        self.assertIsInstance(parsed_list_response, list)
+        for token in parsed_list_response:
+            self.assertEqual(['name', 'token'], sorted(token))
+            self.assertIsInstance(token['name'], str)
+            self.assertIsInstance(token['token'], str)
 
 
 class TestSSHKeyHandlers(APITestCase.ForUser):

=== modified file 'src/maasserver/bootresources.py'
--- src/maasserver/bootresources.py	2016-10-04 13:59:47 +0000
+++ src/maasserver/bootresources.py	2016-10-12 15:26:17 +0000
@@ -316,6 +316,8 @@
             product['kflavor'] = resource.kflavor
         if resource.bootloader_type is not None:
             product['bootloader-type'] = resource.bootloader_type
+        if resource.rolling:
+            product['rolling'] = resource.rolling
         product.update(resource.extra)
         return product
 
@@ -491,7 +493,12 @@
                 )
             )
             if has_kflavor:
-                subarch = "%s-%s" % (subarch, kflavor)
+                if 'edge' in subarch:
+                    subarch_parts = subarch.split('-')
+                    subarch_parts.insert(-1, kflavor)
+                    subarch = '-'.join(subarch_parts)
+                else:
+                    subarch = "%s-%s" % (subarch, kflavor)
             architecture = '%s/%s' % (arch, subarch)
             series = product['release']
         else:
@@ -528,6 +535,7 @@
 
         resource.kflavor = kflavor
         resource.bootloader_type = bootloader_type
+        resource.rolling = product.get('rolling', False)
 
         # Simplestreams content from maas.io includes the following
         # extra fields. Looping through the extra product data and adding it to
@@ -1295,9 +1303,10 @@
     create_gnupg_home()
 
     # Ensure that boot sources exist.
-    if not ensure_boot_source_definition():
-        # Nothing was created so cache the boot sources before import.
-        cache_boot_sources()
+    ensure_boot_source_definition()
+
+    # Cache the boot sources before import.
+    cache_boot_sources()
 
     # FIXME: This modifies the environment of the entire process, which is Not
     # Cool. We should integrate with simplestreams in a more Pythonic manner.

=== modified file 'src/maasserver/dhcp.py'
--- src/maasserver/dhcp.py	2016-09-30 14:50:32 +0000
+++ src/maasserver/dhcp.py	2016-10-12 15:26:17 +0000
@@ -22,6 +22,7 @@
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.db.models import Q
+from maasserver.dns.zonegenerator import get_dns_server_address
 from maasserver.enum import (
     INTERFACE_TYPE,
     IPADDRESS_TYPE,
@@ -444,9 +445,6 @@
         ip_version: int, rack_controller, vlan, subnets: list,
         ntp_servers: Union[list, dict], domain, dhcp_snippets: Iterable=None):
     """Get the DHCP configuration for `ip_version`."""
-    # Circular imports.
-    from maasserver.dns.zonegenerator import get_dns_server_address
-
     try:
         maas_dns_server = get_dns_server_address(
             rack_controller, ipv4=(ip_version == 4), ipv6=(ip_version == 6))

=== modified file 'src/maasserver/dns/tests/test_zonegenerator.py'
--- src/maasserver/dns/tests/test_zonegenerator.py	2016-09-22 02:53:33 +0000
+++ src/maasserver/dns/tests/test_zonegenerator.py	2016-10-12 15:26:17 +0000
@@ -17,7 +17,6 @@
 from maasserver import server_address
 from maasserver.dns import zonegenerator
 from maasserver.dns.zonegenerator import (
-    DNSException,
     get_dns_search_paths,
     get_dns_server_address,
     get_hostname_dnsdata_mapping,
@@ -32,6 +31,7 @@
     NODE_STATUS,
     RDNS_MODE,
 )
+from maasserver.exceptions import UnresolvableHost
 from maasserver.models import (
     Config,
     Domain,
@@ -97,7 +97,7 @@
         self.patch(
             zonegenerator, 'get_maas_facing_server_addresses',
             FakeMethod(failure=socket.error))
-        self.assertRaises(DNSException, get_dns_server_address)
+        self.assertRaises(UnresolvableHost, get_dns_server_address)
 
     def test_get_dns_server_address_logs_warning_if_ip_is_localhost(self):
         logger = self.patch(zonegenerator, 'logger')

=== modified file 'src/maasserver/dns/zonegenerator.py'
--- src/maasserver/dns/zonegenerator.py	2016-09-06 15:08:41 +0000
+++ src/maasserver/dns/zonegenerator.py	2016-10-12 15:26:17 +0000
@@ -17,7 +17,7 @@
     IPRANGE_TYPE,
     RDNS_MODE,
 )
-from maasserver.exceptions import MAASException
+from maasserver.exceptions import UnresolvableHost
 from maasserver.models.config import Config
 from maasserver.models.dnsdata import DNSData
 from maasserver.models.dnsresource import separate_fqdn
@@ -83,10 +83,6 @@
     return DNSData.objects.get_hostname_dnsdata_mapping(domain)
 
 
-class DNSException(MAASException):
-    """An error occured when setting up MAAS's DNS server."""
-
-
 WARNING_MESSAGE = (
     "The DNS server will use the address '%s',  which is inside the "
     "loopback network.  This may not be a problem if you're not using "
@@ -141,7 +137,7 @@
         iplist = get_maas_facing_server_addresses(
             rack_controller, ipv4=ipv4, ipv6=ipv6)
     except socket.error as e:
-        raise DNSException(
+        raise UnresolvableHost(
             "Unable to find MAAS server IP address: %s. MAAS's DNS server "
             "requires this IP address for the NS records in its zone files. "
             "Make sure that the configuration setting for the MAAS URL has "

=== modified file 'src/maasserver/fields.py'
--- src/maasserver/fields.py	2016-09-13 22:22:18 +0000
+++ src/maasserver/fields.py	2016-10-12 15:26:17 +0000
@@ -610,6 +610,10 @@
         looks_like_ip = self.pt_ip.match(value) is not None
         if looks_like_ip:
             return self._clean_addr(value)
+        elif ":" in value:
+            # This is probably an IPv6 address. It's definitely not a
+            # hostname.
+            return self._clean_addr(value)
         else:
             return self._clean_host(value)
 

=== modified file 'src/maasserver/forms_interface_link.py'
--- src/maasserver/forms_interface_link.py	2016-06-29 14:45:14 +0000
+++ src/maasserver/forms_interface_link.py	2016-10-12 15:26:17 +0000
@@ -25,6 +25,7 @@
     BondInterface,
     Interface,
     StaticIPAddress,
+    Subnet,
 )
 from maasserver.utils.forms import (
     compose_invalid_choice_text,
@@ -73,7 +74,11 @@
                 'invalid_choice': compose_invalid_choice_text(
                     'mode', mode_choices),
             })
-        self.fields['subnet'].queryset = self.instance.vlan.subnet_set.all()
+        if self.instance.vlan is None:
+            self.fields['subnet'].queryset = Subnet.objects.none()
+        else:
+            self.fields['subnet'].queryset = (
+                self.instance.vlan.subnet_set.all())
 
     def clean(self):
         for interface_set in self.instance.interface_set.all():

=== modified file 'src/maasserver/forms_settings.py'
--- src/maasserver/forms_settings.py	2016-09-29 16:50:36 +0000
+++ src/maasserver/forms_settings.py	2016-10-12 15:26:17 +0000
@@ -282,10 +282,12 @@
         'form_kwargs': {
             'label': "Addresses of NTP servers",
             'required': False,
-            'help_text': (
-                "NTP servers, specified as IP addresses or hostnames, to be "
-                "used as time references for MAAS itself and the machines "
-                "MAAS deploys, e.g. ntp.ubuntu.com"),
+            'help_text': normalise_whitespace("""\
+                NTP servers, specified as IP addresses or hostnames delimited
+                by commas and/or spaces, to be used as time references for
+                MAAS itself, the machines MAAS deploys, and devices that make
+                use of MAAS's DHCP services.
+            """),
         }
     },
     'ntp_external_only': {

=== modified file 'src/maasserver/management/commands/apikey.py'
--- src/maasserver/management/commands/apikey.py	2016-06-22 17:03:02 +0000
+++ src/maasserver/management/commands/apikey.py	2016-10-12 15:26:17 +0000
@@ -36,17 +36,34 @@
         parser.add_argument(
             '--delete', default=None,
             help="Delete the supplied api key.")
+        parser.add_argument(
+            '--update', default=None,
+            help="Update the supplied api key name")
+        parser.add_argument(
+            '--name', default=None, dest="api_key_name",
+            help="Name of the token. This argument should be passed to "
+                 "--update or --generate options")
+        parser.add_argument(
+            '--with-names ', dest="with_names", action="store_true",
+            help="Display tokens with their names.")
 
     def _print_token(self, token):
-        """Write `token` to stdout in the standard format."""
-        self.stdout.write(convert_tuple_to_string(get_creds_tuple(token)))
+        """Write `token` to stdout in the standard format (with names if
+        --with-names option is enabled)"""
+        if self.display_names:
+            self.stdout.write("%s %s" % (
+                convert_tuple_to_string(get_creds_tuple(token)),
+                token.consumer.name
+            ))
+        else:
+            self.stdout.write(convert_tuple_to_string(get_creds_tuple(token)))
         # In Django 1.5+, self.stdout.write() adds a newline character at
         # the end of the message.
         if django.VERSION < (1, 5):
             self.stdout.write('\n')
 
-    def _generate_token(self, user):
-        _, token = user.userprofile.create_authorisation_token()
+    def _generate_token(self, user, consumer_name=None):
+        _, token = user.userprofile.create_authorisation_token(consumer_name)
         self._print_token(token)
 
     def _delete_token(self, user, key_to_delete):
@@ -61,6 +78,17 @@
         except Http404:
             raise CommandError("No matching api key found.")
 
+    def _update_token(self, user, key_to_update, consumer_name):
+        try:
+            creds_tuple = convert_string_to_tuple(key_to_update)
+        except ValueError as e:
+            raise CommandError(e)
+        _, token_key, _ = creds_tuple
+        try:
+            user.userprofile.modify_consumer_name(token_key, consumer_name)
+        except Http404:
+            raise CommandError("No matching api key found.")
+
     def handle(self, *args, **options):
         username = options.get('username', None)
         if username is None:
@@ -68,16 +96,30 @@
 
         generate = options.get('generate')
         key_to_delete = options.get('delete', None)
+        key_to_update = options.get('update', None)
+        consumer_name = options.get('api_key_name', None)
+        self.display_names = options.get('with_names', None)
+
         if generate and key_to_delete is not None:
             raise CommandError("Specify one of --generate or --delete.")
 
+        if generate and key_to_update is not None:
+            raise CommandError("Specify one of --generate or --update.")
+
+        if key_to_delete and key_to_update is not None:
+            raise CommandError("Specify one of --delete or --update.")
+
+        if key_to_update is not None and consumer_name is None:
+            raise CommandError("Should specify new name for the updated key .")
+
         user = get_one(User.objects.filter(username=username))
+
         if user is None:
             raise CommandError("User does not exist.")
 
         if generate:
             # Generate a new api key.
-            self._generate_token(user)
+            self._generate_token(user, consumer_name)
             return
 
         elif key_to_delete is not None:
@@ -85,6 +127,11 @@
             self._delete_token(user, key_to_delete)
             return
 
+        elif key_to_update is not None:
+            # Update an existing key name.
+            self._update_token(user, key_to_update, consumer_name)
+            return
+
         else:
             # No mutating action requested, so just print existing keys.
             tokens = user.userprofile.get_authorisation_tokens()

=== added file 'src/maasserver/migrations/builtin/maasserver/0092_rolling.py'
--- src/maasserver/migrations/builtin/maasserver/0092_rolling.py	1970-01-01 00:00:00 +0000
+++ src/maasserver/migrations/builtin/maasserver/0092_rolling.py	2016-10-04 15:38:41 +0000
@@ -0,0 +1,22 @@
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals
+
+from django.db import (
+    migrations,
+    models,
+)
+
+
+class Migration(migrations.Migration):
+
+    dependencies = [
+        ('maasserver', '0091_v2_to_v3'),
+    ]
+
+    operations = [
+        migrations.AddField(
+            model_name='bootresource',
+            name='rolling',
+            field=models.BooleanField(default=False),
+        ),
+    ]

=== modified file 'src/maasserver/models/bootresource.py'
--- src/maasserver/models/bootresource.py	2016-10-04 16:07:26 +0000
+++ src/maasserver/models/bootresource.py	2016-10-12 15:26:17 +0000
@@ -9,6 +9,7 @@
 
 from django.core.exceptions import ValidationError
 from django.db.models import (
+    BooleanField,
     CharField,
     Count,
     IntegerField,
@@ -251,6 +252,11 @@
             subarch = resource.split_arch()[1]
             if subarch.startswith("hwe-") or subarch.startswith("ga-"):
                 kernels.add(subarch)
+                if resource.rolling:
+                    subarch_parts = subarch.split('-')
+                    subarch_parts[1] = 'rolling'
+                    kernels.add('-'.join(subarch_parts))
+
             if "subarches" in resource.extra:
                 for subarch in resource.extra["subarches"].split(","):
                     if subarch.startswith("hwe-") or subarch.startswith("ga-"):
@@ -275,6 +281,21 @@
         """Return the kernel package name for the kernel specified."""
         if not node.hwe_kernel:
             return None
+        elif 'hwe-rolling' in node.hwe_kernel:
+            kparts = node.hwe_kernel.split('-')
+            if kparts[-1] == 'edge':
+                if len(kparts) == 3:
+                    kflavor = 'generic'
+                else:
+                    kflavor = kparts[-2]
+                return 'linux-%s-hwe-rolling-edge' % kflavor
+            else:
+                if len(kparts) == 2:
+                    kflavor = 'generic'
+                else:
+                    kflavor = kparts[-1]
+                return 'linux-%s-hwe-rolling' % kflavor
+
         arch = node.split_arch()[0]
         os_release = node.get_osystem() + '/' + node.get_distro_series()
         # Before hwe_kernel was introduced the subarchitecture was the
@@ -372,6 +393,14 @@
 
     kflavor = CharField(max_length=32, blank=True, null=True)
 
+    # The hwe-rolling kernel is a meta-package which depends on the latest
+    # kernel available. Instead of placing a duplicate kernel in the stream
+    # SimpleStreams adds a boolean field to indicate that the hwe-rolling
+    # kernel meta-package points to this kernel. When the rolling field is set
+    # true MAAS allows users to deploy the hwe-rolling kernel by using this
+    # BootResource kernel and instructs Curtin to install the meta-package.
+    rolling = BooleanField(blank=False, null=False, default=False)
+
     extra = JSONObjectField(blank=True, default="", editable=False)
 
     def __str__(self):

=== modified file 'src/maasserver/models/signals/bootsources.py'
--- src/maasserver/models/signals/bootsources.py	2016-06-22 17:03:02 +0000
+++ src/maasserver/models/signals/bootsources.py	2016-10-12 15:26:17 +0000
@@ -9,6 +9,7 @@
 
 from django.db.models.signals import (
     post_delete,
+    post_init,
     post_save,
 )
 from maasserver.bootsources import cache_boot_sources
@@ -21,15 +22,27 @@
 signals = SignalsManager()
 
 
+def is_first_run(sender, instance, *args, **kwargs):
+    """Store whether or not this is the first BootSource added to MAAS.
+
+    On first run the ImportResourceService sets the default BootSource then
+    caches the stream's contents as normal. Setting the default BootSource
+    triggers this signal. This prevents updating the cache twice.
+    """
+    instance.__is_first_run = BootSource.objects.count() == 0
+
+
 def update_boot_source_cache(sender, instance, *args, **kwargs):
     """Update the `BootSourceCache` using the updated source.
 
     This only begins after a successful commit to the database, and is then
     run in a thread. Nothing waits for its completion.
     """
-    post_commit_do(reactor.callLater, 0, cache_boot_sources)
-
-
+    if not isinstance(instance, BootSource) or not instance.__is_first_run:
+        post_commit_do(reactor.callLater, 0, cache_boot_sources)
+
+
+signals.watch(post_init, is_first_run, BootSource)
 signals.watch(post_save, update_boot_source_cache, BootSource)
 signals.watch(post_delete, update_boot_source_cache, BootSource)
 signals.watch_config(update_boot_source_cache, "enable_http_proxy")

=== modified file 'src/maasserver/models/signals/tests/test_bootsources.py'
--- src/maasserver/models/signals/tests/test_bootsources.py	2016-05-25 20:32:57 +0000
+++ src/maasserver/models/signals/tests/test_bootsources.py	2016-10-12 15:26:17 +0000
@@ -10,20 +10,30 @@
 from maasserver.models.config import Config
 from maasserver.testing.factory import factory
 from maasserver.testing.testcase import MAASServerTestCase
-from maastesting.matchers import MockCalledOnceWith
+from maastesting.matchers import (
+    MockCalledOnceWith,
+    MockNotCalled,
+)
 from twisted.internet import reactor
 
 
 class TestBootSourceSignals(MAASServerTestCase):
     """Tests for the `BootSource` model's signals."""
 
+    def test_doesnt_update_on_initial_BootSource_create(self):
+        post_commit_do = self.patch(signals.bootsources, "post_commit_do")
+        factory.make_BootSource(keyring_data=factory.make_bytes())
+        self.assertThat(post_commit_do, MockNotCalled())
+
     def test_arranges_for_update_on_BootSource_create(self):
+        factory.make_BootSource(keyring_data=factory.make_bytes())
         post_commit_do = self.patch(signals.bootsources, "post_commit_do")
         factory.make_BootSource(keyring_data=factory.make_bytes())
         self.assertThat(post_commit_do, MockCalledOnceWith(
             reactor.callLater, 0, cache_boot_sources))
 
     def test_arranges_for_update_on_BootSource_update(self):
+        factory.make_BootSource(keyring_data=factory.make_bytes())
         self.patch(signals.bootsources, "post_commit_do")
         boot_source = factory.make_BootSource(
             keyring_data=factory.make_bytes())
@@ -34,6 +44,7 @@
             reactor.callLater, 0, cache_boot_sources))
 
     def test_arranges_for_update_on_BootSource_delete(self):
+        factory.make_BootSource(keyring_data=factory.make_bytes())
         self.patch(signals.bootsources, "post_commit_do")
         boot_source = factory.make_BootSource(
             keyring_data=factory.make_bytes())

=== modified file 'src/maasserver/models/tests/test_bootresource.py'
--- src/maasserver/models/tests/test_bootresource.py	2016-10-04 16:07:26 +0000
+++ src/maasserver/models/tests/test_bootresource.py	2016-10-12 15:26:17 +0000
@@ -606,19 +606,19 @@
             "name": "ubuntu/trusty",
             "arch": "amd64",
             "subarch": "generic",
-            "kernels": ["hwe-t", "hwe-u", "hwe-v"],
+            "kernels": ["hwe-t", "hwe-u", "hwe-v", "hwe-rolling"],
             }),
         ("ubuntu/vivid", {
             "name": "ubuntu/vivid",
             "arch": "i386",
             "subarch": "generic",
-            "kernels": ["hwe-v"],
+            "kernels": ["hwe-v", "hwe-rolling"],
             }),
         ("ubuntu/precise", {
             "name": "ubuntu/precise",
             "arch": "armfh",
             "subarch": "generic",
-            "kernels": ["hwe-p", "hwe-t", "hwe-v"],
+            "kernels": ["hwe-p", "hwe-t", "hwe-v", "hwe-rolling"],
             }),
         ("ubuntu/wily", {
             "name": "ubuntu/wily",
@@ -630,7 +630,9 @@
             "name": "ubuntu/xenial",
             "arch": "amd64",
             "subarch": "generic",
-            "kernels": ["hwe-16.04", "hwe-16.04-lowlatency"],
+            "kernels": [
+                "hwe-16.04", "hwe-16.04-lowlatency", "hwe-rolling",
+                "hwe-rolling-lowlatency"],
             }),
         )
 
@@ -646,7 +648,8 @@
                     generic_kernels.append(i)
                 factory.make_usable_boot_resource(
                     name=self.name, rtype=BOOT_RESOURCE_TYPE.SYNCED,
-                    architecture="%s/%s" % (self.arch, i), kflavor=kflavor)
+                    architecture="%s/%s" % (self.arch, i), kflavor=kflavor,
+                    rolling=True)
                 factory.make_incomplete_boot_resource(
                     name=self.name, rtype=BOOT_RESOURCE_TYPE.SYNCED,
                     architecture="%s/%s" % (
@@ -656,7 +659,7 @@
             generic_kernels = self.kernels
             factory.make_usable_boot_resource(
                 name=self.name, rtype=BOOT_RESOURCE_TYPE.SYNCED,
-                architecture="%s/%s" % (self.arch, self.subarch))
+                architecture="%s/%s" % (self.arch, self.subarch), rolling=True)
         self.assertEqual(
             sorted(self.kernels),
             sorted(BootResource.objects.get_usable_hwe_kernels(
@@ -690,6 +693,23 @@
             "linux-image-generic-lts-trusty",
             BootResource.objects.get_kpackage_for_node(node))
 
+    def test__returns_hwe_rolling(self):
+        node = factory.make_Node()
+        for hwe_kernel, kpackage in (
+                ['hwe-rolling', 'linux-generic-hwe-rolling'],
+                ['hwe-rolling-lowlatency', 'linux-lowlatency-hwe-rolling'],
+                ['hwe-rolling-edge', 'linux-generic-hwe-rolling-edge'],
+                [
+                    'hwe-rolling-lowlatency-edge',
+                    'linux-lowlatency-hwe-rolling-edge'],):
+            node.hwe_kernel = hwe_kernel
+            self.assertEqual(
+                kpackage, BootResource.objects.get_kpackage_for_node(node))
+
+    def test__returns_none(self):
+        node = factory.make_Node()
+        self.assertIsNone(BootResource.objects.get_kpackage_for_node(node))
+
 
 class TestBootResource(MAASServerTestCase):
     """Tests for the `BootResource` model."""

=== modified file 'src/maasserver/models/user.py'
--- src/maasserver/models/user.py	2015-12-02 20:43:30 +0000
+++ src/maasserver/models/user.py	2016-10-12 15:26:17 +0000
@@ -29,17 +29,21 @@
 GENERIC_CONSUMER = 'MAAS consumer'
 
 
-def create_auth_token(user):
+def create_auth_token(user, consumer_name=None):
     """Create new Token and Consumer (OAuth authorisation) for `user`.
 
     :param user: The user to create a token for.
     :type user: User
+    :param consumer_name: Name of the consumer to be assigned to the newly
+     generated token.
     :return: The created Token.
     :rtype: piston.models.Token
 
     """
+    if consumer_name is None:
+        consumer_name = GENERIC_CONSUMER
     consumer = Consumer.objects.create(
-        user=user, name=GENERIC_CONSUMER, status='accepted')
+        user=user, name=consumer_name, status='accepted')
     consumer.generate_random_codes()
     # This is a 'generic' consumer aimed to service many clients, hence
     # we don't authenticate the consumer with key/secret key.

=== modified file 'src/maasserver/models/userprofile.py'
--- src/maasserver/models/userprofile.py	2016-09-22 02:53:33 +0000
+++ src/maasserver/models/userprofile.py	2016-10-12 15:26:17 +0000
@@ -92,7 +92,7 @@
 
         return get_auth_tokens(self.user)
 
-    def create_authorisation_token(self):
+    def create_authorisation_token(self, consumer_name=None):
         """Create a new Token and its related Consumer (OAuth authorisation).
 
         :return: A tuple containing the Consumer and the Token that were
@@ -103,7 +103,7 @@
         # Avoid circular imports.
         from maasserver.models.user import create_auth_token
 
-        token = create_auth_token(self.user)
+        token = create_auth_token(self.user, consumer_name)
         return token.consumer, token
 
     def delete_authorisation_token(self, token_key):
@@ -119,5 +119,20 @@
         token.consumer.delete()
         token.delete()
 
+    def modify_consumer_name(self, token_key, consumer_name):
+        """Modify consumer name of an existing token key.
+
+        :param token_key: The key of the token to be deleted.
+        :type token_key: string
+        :param consumer_name: Name of the token consumer.
+        :type consumer_name: string
+        :raises: `django.http.Http404`
+        """
+        token = get_object_or_404(
+            Token, user=self.user, token_type=Token.ACCESS, key=token_key)
+        token.consumer.name = consumer_name
+        token.consumer.save()
+        token.save()
+
     def __str__(self):
         return self.user.username

=== modified file 'src/maasserver/populate_tags.py'
--- src/maasserver/populate_tags.py	2016-05-09 18:31:31 +0000
+++ src/maasserver/populate_tags.py	2016-10-12 15:26:17 +0000
@@ -119,8 +119,7 @@
                     "credentials": creds,
                     "nodes": list(chunked_node_ids[idx]),
                 })
-        [d] = _do_populate_tags(connected_racks)
-        return d
+        return _do_populate_tags(connected_racks)
 
 
 def _get_or_create_auth_token(user):

=== modified file 'src/maasserver/rpc/rackcontrollers.py'
--- src/maasserver/rpc/rackcontrollers.py	2016-09-28 19:08:40 +0000
+++ src/maasserver/rpc/rackcontrollers.py	2016-10-12 15:26:17 +0000
@@ -18,12 +18,14 @@
 from maasserver import (
     locks,
     worker_user,
+    is_master_process,
 )
 from maasserver.enum import NODE_TYPE
 from maasserver.models import (
     Node,
     NodeGroupToRackController,
     RackController,
+    RegionController,
     StaticIPAddress,
 )
 from maasserver.models.node import typecast_node
@@ -91,21 +93,31 @@
     if interfaces is None:
         interfaces = {}
 
+    this_region = RegionController.objects.get_running_controller()
     node = find(system_id, hostname, interfaces)
     if node is None:
         node = RackController.objects.create(hostname=hostname)
-        maaslog.info("Created new rack controller '%s'.", node.hostname)
+        maaslog.info(
+            "New rack controller '%s' was created by region '%s' upon "
+            "first connection.",
+            node.hostname, this_region.hostname)
     elif node.is_rack_controller:
-        maaslog.info(
-            "Registering existing rack controller '%s'.", node.hostname)
+        if is_master_process():
+            # Only the master process logs to the maaslog.
+            maaslog.info(
+                "Existing rack controller '%s' has connected to region '%s'.",
+                node.hostname, this_region.hostname)
     elif node.is_region_controller:
         maaslog.info(
-            "Converting '%s' into a region and rack controller.",
+            "Region controller '%s' converted into a region and rack "
+            "controller.",
             node.hostname)
         node.node_type = NODE_TYPE.REGION_AND_RACK_CONTROLLER
         node.save()
     else:
-        maaslog.info("Converting '%s' into a rack controller.", node.hostname)
+        maaslog.info(
+            "Region controller '%s' converted '%s' into a rack controller.",
+            this_region.hostname, node.hostname)
         node.node_type = NODE_TYPE.RACK_CONTROLLER
         node.save()
 

=== modified file 'src/maasserver/rpc/regionservice.py'
--- src/maasserver/rpc/regionservice.py	2016-10-03 20:36:58 +0000
+++ src/maasserver/rpc/regionservice.py	2016-10-12 15:26:17 +0000
@@ -656,7 +656,8 @@
 
             # Rack controller is now registered. Log this status.
             log.msg(
-                "Rack controller '%s' has been registered." % self.ident)
+                "Process [%s] - registered rack controller '%s'." % (
+                    os.getpid(), self.ident))
 
             # Done registering the rack controller and connection.
             return {'system_id': self.ident}

=== modified file 'src/maasserver/rpc/tests/test_rackcontrollers.py'
--- src/maasserver/rpc/tests/test_rackcontrollers.py	2016-09-28 19:09:32 +0000
+++ src/maasserver/rpc/tests/test_rackcontrollers.py	2016-10-12 15:26:17 +0000
@@ -23,6 +23,7 @@
     Node,
     NodeGroupToRackController,
     RackController,
+    RegionController,
 )
 from maasserver.models.interface import PhysicalInterface
 from maasserver.models.timestampedmodel import now
@@ -111,6 +112,14 @@
 
 class TestRegisterRackController(MAASServerTestCase):
 
+    def setUp(self):
+        super(TestRegisterRackController, self).setUp()
+        self.this_region = factory.make_RegionController()
+        mock_running = self.patch(
+            RegionController.objects,
+            "get_running_controller")
+        mock_running.return_value = self.this_region
+
     def test_sets_owner_to_worker_when_none(self):
         node = factory.make_Node()
         rack_registered = register(system_id=node.system_id)
@@ -155,12 +164,14 @@
         register(system_id=node.system_id)
         self.assertEqual(node_type, node.node_type)
 
-    def test_logs_finding_existing_node(self):
+    def test_logs_finding_existing_node_when_master(self):
         logger = self.useFixture(FakeLogger("maas"))
         node = factory.make_Node(node_type=NODE_TYPE.RACK_CONTROLLER)
+        self.patch(rackcontrollers, "is_master_process").return_value = True
         register(system_id=node.system_id)
         self.assertEqual(
-            "Registering existing rack controller '%s'." % node.hostname,
+            "Existing rack controller '%s' has connected to region '%s'." % (
+                node.hostname, self.this_region.hostname),
             logger.output.strip())
 
     def test_converts_region_controller(self):
@@ -174,7 +185,8 @@
         node = factory.make_Node(node_type=NODE_TYPE.REGION_CONTROLLER)
         register(system_id=node.system_id)
         self.assertEqual(
-            "Converting '%s' into a region and rack controller.\n" %
+            "Region controller '%s' converted into a region and "
+            "rack controller.\n" %
             node.hostname, logger.output)
 
     def test_converts_existing_node(self):
@@ -187,7 +199,8 @@
         node = factory.make_Node(node_type=NODE_TYPE.MACHINE)
         register(system_id=node.system_id)
         self.assertEqual(
-            "Converting '%s' into a rack controller.\n" % node.hostname,
+            "Region controller '%s' converted '%s' into a rack "
+            "controller.\n" % (self.this_region.hostname, node.hostname),
             logger.output)
 
     def test_creates_new_rackcontroller(self):
@@ -210,7 +223,9 @@
         hostname = factory.make_name("hostname")
         register(hostname=hostname)
         self.assertEqual(
-            "Created new rack controller '%s'." % hostname,
+            "New rack controller '%s' was created by region '%s' upon first "
+            "connection." % (
+                hostname, self.this_region.hostname),
             logger.output.strip())
 
     def test_sets_interfaces(self):

=== modified file 'src/maasserver/rpc/tests/test_regionservice.py'
--- src/maasserver/rpc/tests/test_regionservice.py	2016-08-09 19:32:35 +0000
+++ src/maasserver/rpc/tests/test_regionservice.py	2016-10-12 15:26:17 +0000
@@ -381,11 +381,17 @@
             RegisterRackController.commandName)
         self.assertIsNotNone(responder)
 
+    @inlineCallbacks
     def installFakeRegionAdvertisingService(self):
+        region = yield deferToDatabase(
+            transactional(factory.make_RegionController))
+        self.patch(
+            RegionController.objects,
+            "get_running_controller").return_value = region
         service = RegionAdvertisingService()
         service.setName("rpc-advertise")
         service.advertising.set(RegionAdvertising(
-            region_id=factory.make_name("region-id"),
+            region_id=region.system_id,
             process_id=randint(1000, 9999)))
         service.setServiceParent(eventloop.services)
         self.addCleanup(service.disownServiceParent)
@@ -393,7 +399,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_returns_system_id(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -409,7 +415,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_updates_interfaces(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -440,7 +446,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_calls_handle_upgrade(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -460,7 +466,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_sets_ident(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -475,7 +481,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_calls_addConnectionFor(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -494,7 +500,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_sets_hosts(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -510,7 +516,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_sets_hostIsRemote_calls_registerConnection(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         rack_controller = yield deferToDatabase(factory.make_RackController)
         protocol = self.make_Region()
         protocol.transport = MagicMock()
@@ -537,7 +543,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_creates_new_rack(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         protocol = self.make_Region()
         protocol.transport = MagicMock()
         hostname = factory.make_hostname()
@@ -553,7 +559,7 @@
     @wait_for_reactor
     @inlineCallbacks
     def test_register_raises_CannotRegisterRackController_when_it_cant(self):
-        self.installFakeRegionAdvertisingService()
+        yield self.installFakeRegionAdvertisingService()
         patched_create = self.patch(RackController.objects, 'create')
         patched_create.side_effect = IntegrityError()
         hostname = factory.make_name("hostname")

=== modified file 'src/maasserver/static/js/angular/controllers/networks_list.js'
--- src/maasserver/static/js/angular/controllers/networks_list.js	2016-09-21 14:49:23 +0000
+++ src/maasserver/static/js/angular/controllers/networks_list.js	2016-10-12 15:26:17 +0000
@@ -18,7 +18,7 @@
         var filterBySpace = $filter('filterBySpace');
 
         // Set title and page.
-        $rootScope.title = "Networks";
+        $rootScope.title = "Subnets";
         $rootScope.page = "networks";
 
         // Set the initial value of $scope.groupBy based on the URL

=== modified file 'src/maasserver/static/js/angular/controllers/tests/test_networks_list.js'
--- src/maasserver/static/js/angular/controllers/tests/test_networks_list.js	2016-09-22 15:05:38 +0000
+++ src/maasserver/static/js/angular/controllers/tests/test_networks_list.js	2016-10-12 15:26:17 +0000
@@ -60,7 +60,7 @@
 
     it("sets title and page on $rootScope", function() {
         var controller = makeController();
-        expect($rootScope.title).toBe("Networks");
+        expect($rootScope.title).toBe("Subnets");
         expect($rootScope.page).toBe("networks");
     });
 

=== modified file 'src/maasserver/static/js/angular/directives/boot_images.js'
--- src/maasserver/static/js/angular/directives/boot_images.js	2016-10-03 16:19:07 +0000
+++ src/maasserver/static/js/angular/directives/boot_images.js	2016-10-12 15:26:17 +0000
@@ -378,7 +378,7 @@
                                         title: release.title,
                                         arch: arch.title,
                                         size: '-',
-                                        status: 'Queued for download',
+                                        status: 'Selected for download',
                                         beingDeleted: false
                                     };
                                     var resource = getResource(
@@ -406,7 +406,7 @@
                                 title: resource.title,
                                 arch: resource.arch,
                                 size: resource.size,
-                                status: 'Queued for deletion',
+                                status: 'Will be deleted',
                                 beingDeleted: true
                             };
                             $scope.ubuntuImages.push(image);
@@ -454,7 +454,7 @@
                                     title: otherImage.title,
                                     arch: name_split[1],
                                     size: '-',
-                                    status: 'Queued for download',
+                                    status: 'Selected for download',
                                     beingDeleted: false
                                 };
                                 var resource = getResource(
@@ -480,7 +480,7 @@
                             title: resource.title,
                             arch: resource.arch,
                             size: resource.size,
-                            status: 'Queued for deletion',
+                            status: 'Will be deleted',
                             beingDeleted: true
                         };
                         $scope.otherImages.push(image);

=== modified file 'src/maasserver/static/js/angular/directives/tests/test_boot_images.js'
--- src/maasserver/static/js/angular/directives/tests/test_boot_images.js	2016-10-03 16:19:07 +0000
+++ src/maasserver/static/js/angular/directives/tests/test_boot_images.js	2016-10-12 15:26:17 +0000
@@ -972,7 +972,7 @@
                 title: release.title,
                 arch: arch.title,
                 size: '-',
-                status: 'Queued for download',
+                status: 'Selected for download',
                 beingDeleted: false
             }]);
         });
@@ -1048,7 +1048,7 @@
                 title: release.title,
                 arch: arch.name,
                 size: size,
-                status: 'Queued for deletion',
+                status: 'Will be deleted',
                 beingDeleted: true
             }]);
         });
@@ -1076,7 +1076,7 @@
                 title: image.title,
                 arch: arch,
                 size: '-',
-                status: 'Queued for download',
+                status: 'Selected for download',
                 beingDeleted: false
             }]);
         });
@@ -1149,7 +1149,7 @@
                 title: image.title,
                 arch: arch,
                 size: size,
-                status: 'Queued for deletion',
+                status: 'Will be deleted',
                 beingDeleted: true
             }]);
         });

=== modified file 'src/maasserver/static/js/angular/maas.js'
--- src/maasserver/static/js/angular/maas.js	2016-09-23 18:48:54 +0000
+++ src/maasserver/static/js/angular/maas.js	2016-10-12 15:26:17 +0000
@@ -9,10 +9,15 @@
  */
 
 angular.module('MAAS', ['ngRoute', 'ngCookies', 'ngTagsInput']).config(
-    function($interpolateProvider, $routeProvider) {
+    function($interpolateProvider, $routeProvider, $httpProvider) {
         $interpolateProvider.startSymbol('{$');
         $interpolateProvider.endSymbol('$}');
 
+        // Set the $httpProvider to send the csrftoken in the header of any
+        // http request.
+        $httpProvider.defaults.xsrfCookieName = 'csrftoken';
+        $httpProvider.defaults.xsrfHeaderName = 'X-CSRFToken';
+
         // Helper that wrappers the templateUrl to append the files version
         // to the path. Used to override client cache.
         function versionedPath(path) {

=== modified file 'src/maasserver/static/partials/boot-images.html'
--- src/maasserver/static/partials/boot-images.html	2016-10-04 13:59:06 +0000
+++ src/maasserver/static/partials/boot-images.html	2016-10-12 15:26:17 +0000
@@ -116,7 +116,7 @@
                             <div class="table__header table-col--25">Release</div>
                             <div class="table__header table-col--15">Architecture</div>
                             <div class="table__header table-col--20">Size</div>
-                            <div class="table__header table-col--30">Importing status</div>
+                            <div class="table__header table-col--30">Status</div>
                         </div>
                     </header>
                     <main class="table__body">
@@ -192,7 +192,7 @@
                     <div class="table__header table-col--25">Release</div>
                     <div class="table__header table-col--15">Architecture</div>
                     <div class="table__header table-col--20">Size</div>
-                    <div class="table__header table-col--30">Importing status</div>
+                    <div class="table__header table-col--30">Status</div>
                 </div>
             </header>
             <main class="table__body">
@@ -233,7 +233,7 @@
                     <div class="table__header table-col--25">Release</div>
                     <div class="table__header table-col--15">Architecture</div>
                     <div class="table__header table-col--20">Size</div>
-                    <div class="table__header table-col--30">Importing status</div>
+                    <div class="table__header table-col--30">Status</div>
                 </div>
             </header>
             <main class="table__body">
@@ -266,7 +266,7 @@
                     <div class="table__header table-col--25">Release</div>
                     <div class="table__header table-col--15">Architecture</div>
                     <div class="table__header table-col--20">Size</div>
-                    <div class="table__header table-col--30">Importing status</div>
+                    <div class="table__header table-col--30">Status</div>
                 </div>
             </header>
             <main class="table__body">

=== modified file 'src/maasserver/static/partials/dashboard.html'
--- src/maasserver/static/partials/dashboard.html	2016-10-05 15:17:51 +0000
+++ src/maasserver/static/partials/dashboard.html	2016-10-12 15:26:17 +0000
@@ -35,8 +35,8 @@
             <div class="table">
                 <header class="table__head">
                     <div class="table__row">
-                        <div class="table__header table-col--20">Name</div>
-                        <div class="table__header table-col--20">
+                        <div class="table__header table-col--37">Name</div>
+                        <div class="table__header table-col--15">
                             <a class="table__header-link"
                                data-ng-click="column = 'mac'"
                                data-ng-class="{'is-active': column === 'mac'}">Mac</a>
@@ -45,8 +45,8 @@
                                data-ng-click="column = 'manufacturer'"
                                data-ng-class="{'is-active': column === 'manufacturer'}">Manufacturer</a>
                         </div>
-                        <div class="table__header table-col--20">IP</div>
-                        <div class="table__header table-col--20">Rack</div>
+                        <div class="table__header table-col--13">IP</div>
+                        <div class="table__header table-col--15">Rack</div>
                         <div class="table__header table-col--20">Last seen</div>
                     </div>
                 </header>
@@ -72,23 +72,23 @@
                         data-ng-class="{'is-active' : discovery.first_seen === selectedDevice}">
                         <div data-ng-if="discovery.first_seen !== selectedDevice"
                             data-ng-dblclick="toggleSelected(discovery.first_seen)">
-                            <div class="table__data table-col--20">
+                            <div class="table__data table-col--37">
                                 {$ getDiscoveryName(discovery) $}
                                 <i data-ng-show="discovery.is_external_dhcp === true" class="icon icon--info tooltip u-margin--left-tiny ng-hide" aria-label="This device is providing DHCP."></i>
                             </div>
-                            <div class="table__data table-col--20" data-ng-if="column === 'mac'">
+                            <div class="table__data table-col--15" data-ng-if="column === 'mac'">
                                 {$ discovery.mac_address $}
                             </div>
-                            <div class="table__data table-col--20" data-ng-if="column === 'manufacturer'">
+                            <div class="table__data table-col--15" data-ng-if="column === 'manufacturer'">
                                 {$ discovery.mac_organization || 'Unknown' $}
                             </div>
-                            <div class="table__data table-col--20 u-text--truncate">
+                            <div class="table__data table-col--13 u-text--truncate">
                                 {$ discovery.ip $}
                             </div>
+                            <div class="table__data table-col--15">
+                                {$ discovery.observer_hostname $}
+                            </div>
                             <div class="table__data table-col--17">
-                                {$ discovery.observer_hostname $}
-                            </div>
-                            <div class="table__data table-col--20">
                                 <time>{$ discovery.last_seen $}</time>
                             </div>
                             <div class="table__data table-col--3">

=== modified file 'src/maasserver/static/partials/nodes-list.html'
--- src/maasserver/static/partials/nodes-list.html	2016-10-04 20:50:58 +0000
+++ src/maasserver/static/partials/nodes-list.html	2016-10-12 15:26:17 +0000
@@ -695,7 +695,7 @@
                     <tr>
                         <th class="table-col--3">
                             <!-- XXX ricgard 2016-06-16 - Need to add e2e test. -->
-                            <input class="checkbox" type="checkbox" data-ng-click="toggleCheckAll('nodes')" data-ng-model="tabs.nodes.allViewableChecked" id="check-all" data-ng-disabled="hasActionsInProgress('nodes')" />
+                            <input class="checkbox" type="checkbox" data-ng-click="toggleCheckAll('nodes')" data-ng-checked="tabs.nodes.allViewableChecked" id="check-all" data-ng-disabled="hasActionsInProgress('nodes')" />
                             <label class="checkbox-label" for="check-all"></label>
                         </th>
                         <th colspan="2" class="u-align--left">
@@ -746,7 +746,7 @@
                         data-ng-class="{ 'table--error': !supportsAction(node, 'nodes'), selected: node.$selected }">
                         <td class="table-col--3">
                             <!-- XXX ricgard 2016-06-16 - Need to add e2e test. -->
-                            <input class="checkbox" type="checkbox" data-ng-click="toggleChecked(node, 'nodes')" data-ng-model="node.$selected" id="{$ node.fqdn $}" data-ng-disabled="hasActionsInProgress('nodes')" />
+                            <input class="checkbox" type="checkbox" data-ng-click="toggleChecked(node, 'nodes')" data-ng-checked="node.$selected" id="{$ node.fqdn $}" data-ng-disabled="hasActionsInProgress('nodes')" />
                             <label class="checkbox-label" for="{$ node.fqdn $}"></label>
                         </td>
                         <!-- XXX ricgard 2016-06-16 - Need to add e2e test. -->
@@ -790,7 +790,7 @@
                 <tr>
                     <th class="table-col--3">
                         <!-- XXX rvba 2015-02-25 - Need to add e2e test. -->
-                        <input type="checkbox" class="checkbox" data-ng-click="toggleCheckAll('devices')" data-ng-model="tabs.devices.allViewableChecked" id="check-all-devices" data-ng-disabled="hasActionsInProgress('devices')" />
+                        <input type="checkbox" class="checkbox" data-ng-click="toggleCheckAll('devices')" data-ng-checked="tabs.devices.allViewableChecked" id="check-all-devices" data-ng-disabled="hasActionsInProgress('devices')" />
                         <label for="check-all-devices" class="checkbox-label"></label>
                     </th>
                     <th class="table-col--24">
@@ -822,7 +822,7 @@
                     data-ng-repeat="device in tabs.devices.filtered_items = (devices | nodesFilter:tabs.devices.search | orderBy:tabs.devices.predicate:tabs.devices.reverse) track by device.system_id"
                     data-ng-class="{ 'table--error': !supportsAction(device, 'devices'), selected: device.$selected }">
                     <td class="table-col--3">
-                        <input type="checkbox" class="checkbox" data-ng-click="toggleChecked(device, 'devices')" data-ng-model="device.$selected" id="{$ device.fqdn $}" data-ng-disabled="hasActionsInProgress('devices')"/>
+                        <input type="checkbox" class="checkbox" data-ng-click="toggleChecked(device, 'devices')" data-ng-checked="device.$selected" id="{$ device.fqdn $}" data-ng-disabled="hasActionsInProgress('devices')"/>
                         <label for="{$ device.fqdn $}" class="checkbox-label"></label>
                     </td>
                     <td class="table-col--24">{$ device.fqdn $}</td>
@@ -854,7 +854,7 @@
                 <tr>
                     <th class="table-col--2">
                         <!-- XXX rvba 2015-02-25 - Need to add e2e test. -->
-                        <input type="checkbox" class="checkbox" data-ng-click="toggleCheckAll('controllers')" data-ng-model="tabs.controllers.allViewableChecked" id="check-all-controllers" data-ng-disabled="hasActionsInProgress('controllers')" />
+                        <input type="checkbox" class="checkbox" data-ng-click="toggleCheckAll('controllers')" data-ng-checked="tabs.controllers.allViewableChecked" id="check-all-controllers" data-ng-disabled="hasActionsInProgress('controllers')" />
                         <label for="check-all-controllers" class="checkbox-label"></label>
                     </th>
                     <th class="table-col--19">
@@ -884,7 +884,7 @@
                     data-ng-repeat="controller in tabs.controllers.filtered_items = (controllers | nodesFilter:tabs.controllers.search | orderBy:tabs.controllers.predicate:tabs.controllers.reverse) track by controller.system_id"
                     data-ng-class="{ 'table--error': !supportsAction(controller, 'controllers'), selected: controller.$selected }">
                     <td class="table-col--2">
-                        <input type="checkbox" class="checkbox" data-ng-click="toggleChecked(controller, 'controllers')" data-ng-model="controller.$selected" id="{$ controller.fqdn $}" data-ng-disabled="hasActionsInProgress('controllers')"/>
+                        <input type="checkbox" class="checkbox" data-ng-click="toggleChecked(controller, 'controllers')" data-ng-checked="controller.$selected" id="{$ controller.fqdn $}" data-ng-disabled="hasActionsInProgress('controllers')"/>
                         <label for="{$ controller.fqdn $}" class="checkbox-label"></label>
                     </td>
                     <td class="table-col--19">

=== modified file 'src/maasserver/testing/factory.py'
--- src/maasserver/testing/factory.py	2016-10-05 15:47:15 +0000
+++ src/maasserver/testing/factory.py	2016-10-12 15:26:17 +0000
@@ -1458,8 +1458,9 @@
             sha256=sha256, size=len(content),
             total_size=size, content=largeobject)
 
-    def make_BootResource(self, rtype=None, name=None, architecture=None,
-                          extra=None, kflavor=None, bootloader_type=None):
+    def make_BootResource(
+            self, rtype=None, name=None, architecture=None, extra=None,
+            kflavor=None, bootloader_type=None, rolling=False):
         if rtype is None:
             rtype = self.pick_enum(BOOT_RESOURCE_TYPE)
         if name is None:
@@ -1480,7 +1481,7 @@
                 }
         return BootResource.objects.create(
             rtype=rtype, name=name, architecture=architecture, kflavor=kflavor,
-            bootloader_type=bootloader_type, extra=extra)
+            bootloader_type=bootloader_type, extra=extra, rolling=rolling)
 
     def make_BootResourceSet(self, resource, version=None, label=None):
         if version is None:
@@ -1518,10 +1519,10 @@
     def make_usable_boot_resource(
             self, rtype=None, name=None, architecture=None,
             extra=None, version=None, label=None, kflavor=None, size=None,
-            bootloader_type=None):
+            bootloader_type=None, rolling=False):
         resource = self.make_BootResource(
             rtype=rtype, name=name, architecture=architecture, extra=extra,
-            kflavor=kflavor, bootloader_type=bootloader_type)
+            kflavor=kflavor, bootloader_type=bootloader_type, rolling=rolling)
         resource_set = self.make_BootResourceSet(
             resource, version=version, label=label)
         filetypes = {

=== modified file 'src/maasserver/tests/test_bootresources.py'
--- src/maasserver/tests/test_bootresources.py	2016-10-04 13:59:47 +0000
+++ src/maasserver/tests/test_bootresources.py	2016-10-12 15:26:17 +0000
@@ -205,9 +205,9 @@
         return 'maas:boot:%s:%s:%s:%s' % (os, arch, subarch, series)
 
     def make_usable_product_boot_resource(
-            self, kflavor=None, bootloader_type=None):
+            self, kflavor=None, bootloader_type=None, rolling=False):
         resource = factory.make_usable_boot_resource(
-            kflavor=kflavor, bootloader_type=bootloader_type)
+            kflavor=kflavor, bootloader_type=bootloader_type, rolling=rolling)
         return self.get_product_name_for_resource(resource), resource
 
     def test_streams_other_than_allowed_returns_404(self):
@@ -351,12 +351,13 @@
         kflavor = factory.make_name('kflavor')
         bootloader_type = factory.make_name('bootloader_type')
         product, _ = self.make_usable_product_boot_resource(
-            kflavor, bootloader_type)
+            kflavor, bootloader_type, True)
         response = self.get_stream_client('maas:v2:download.json')
         output = json.loads(response.content.decode(settings.DEFAULT_CHARSET))
         self.assertEquals(kflavor, output['products'][product]['kflavor'])
         self.assertEquals(
             bootloader_type, output['products'][product]['bootloader-type'])
+        self.assertTrue(output['products'][product]['rolling'])
 
     def test_streams_product_download_product_has_valid_values(self):
         product, resource = self.make_usable_product_boot_resource()
@@ -638,6 +639,7 @@
         'kpackage': factory.make_name('kpackage'),
         'item_name': name,
         'path': '/path/to/%s' % name,
+        'rolling': factory.pick_bool(),
         }
     name = '%s/%s' % (product['os'], product['release'])
     if kflavor == 'generic':
@@ -736,6 +738,7 @@
         self.assertEqual(architecture, resource.architecture)
         self.assertEqual(product['kflavor'], resource.kflavor)
         self.assertEqual(product['subarches'], resource.extra['subarches'])
+        self.assertEqual(product['rolling'], resource.rolling)
 
     def test_get_or_create_boot_resource_handles_bootloader(self):
         osystem = factory.make_name('os')

=== modified file 'src/maasserver/tests/test_commands.py'
--- src/maasserver/tests/test_commands.py	2016-09-23 18:03:06 +0000
+++ src/maasserver/tests/test_commands.py	2016-10-12 15:26:17 +0000
@@ -427,3 +427,59 @@
             self, 'apikey', username=user.username, delete=token_string)
         self.assertIn(
             "No matching api key found", error_text)
+
+    def test_success_modify_apikey_name(self):
+        stderr = StringIO()
+        stdout = StringIO()
+        fake_api_key_name = "Test Key Name"
+        user = factory.make_User()
+        existing_token = get_one(
+            user.userprofile.get_authorisation_tokens())
+        token_string = convert_tuple_to_string(
+            get_creds_tuple(existing_token))
+        call_command(
+            'apikey', username=user.username, update=token_string,
+            api_key_name=fake_api_key_name, stderr=stderr, stdout=stdout)
+        self.assertThat(stderr, IsEmpty)
+
+    def test_api_key_rejects_update_of_nonexistent_key(self):
+        stderr = StringIO()
+        user = factory.make_User()
+        fake_api_key_name = "Test Key Name"
+        existing_token = get_one(
+            user.userprofile.get_authorisation_tokens())
+        token_string = convert_tuple_to_string(
+            get_creds_tuple(existing_token))
+        call_command(
+            'apikey', username=user.username, delete=token_string,
+            stderr=stderr)
+        self.assertThat(stderr, IsEmpty)
+
+        # Try to update the deleted token.
+        error_text = assertCommandErrors(
+            self, 'apikey', username=user.username, update=token_string,
+            api_key_name=fake_api_key_name)
+        self.assertIn(
+            "No matching api key found", error_text)
+
+    def test_api_key_rejects_update_without_key_name(self):
+        user = factory.make_User()
+        existing_token = get_one(
+            user.userprofile.get_authorisation_tokens())
+        token_string = convert_tuple_to_string(
+            get_creds_tuple(existing_token))
+        error_text = assertCommandErrors(
+            self, 'apikey', username=user.username, update=token_string)
+        self.assertIn("Should specify new name", error_text)
+
+    def test_api_key_update_and_generate_mutually_exclusive_options(self):
+        user = factory.make_User()
+        fake_api_key_name = "Test Key Name"
+        existing_token = get_one(
+            user.userprofile.get_authorisation_tokens())
+        token_string = convert_tuple_to_string(
+            get_creds_tuple(existing_token))
+        error_text = assertCommandErrors(
+            self, 'apikey', username=user.username, generate=True,
+            api_key_name=fake_api_key_name, update=token_string)
+        self.assertIn("Specify one of --generate or --update.", error_text)

=== modified file 'src/maasserver/tests/test_fields.py'
--- src/maasserver/tests/test_fields.py	2016-08-25 19:31:20 +0000
+++ src/maasserver/tests/test_fields.py	2016-10-12 15:26:17 +0000
@@ -599,6 +599,19 @@
         input = ','.join(hostnames)
         self.assertEqual(' '.join(hostnames), HostListFormField().clean(input))
 
+    def test_accepts_misc(self):
+        servers = {
+            "::1",
+            "1::",
+            "1::2",
+            "1:2::3",
+            "1::2:3",
+            "1:2::3:4",
+            "::127.0.0.1",
+        }
+        input = ','.join(servers)
+        self.assertEqual(' '.join(servers), HostListFormField().clean(input))
+
     def test_rejects_invalid_ipv4_address(self):
         input = "%s 12.34.56.999" % factory.make_hostname()
         error = self.assertRaises(

=== modified file 'src/maasserver/tests/test_forms_interface_link.py'
--- src/maasserver/tests/test_forms_interface_link.py	2016-06-29 14:45:14 +0000
+++ src/maasserver/tests/test_forms_interface_link.py	2016-10-12 15:26:17 +0000
@@ -43,6 +43,13 @@
         })
         self.assertTrue(form.is_valid(), form.errors)
 
+    def test__sets_subnet_queryset_to_empty_on_interface_wihtout_vlan(self):
+        interface = factory.make_Interface(INTERFACE_TYPE.PHYSICAL)
+        interface.vlan = None
+        interface.save()
+        form = InterfaceLinkForm(instance=interface, data={})
+        self.assertItemsEqual([], form.fields["subnet"].queryset)
+
     def test__sets_subnet_queryset_to_subnets_on_interface_vlan(self):
         interface = factory.make_Interface(INTERFACE_TYPE.PHYSICAL)
         subnets = [

=== modified file 'src/maasserver/tests/test_populate_tags.py'
--- src/maasserver/tests/test_populate_tags.py	2016-06-21 10:29:11 +0000
+++ src/maasserver/tests/test_populate_tags.py	2016-10-12 15:26:17 +0000
@@ -233,7 +233,7 @@
             protocols.append(protocol)
         tag = factory.make_Tag(populate=False)
 
-        d = populate_tags(tag)
+        [d] = populate_tags(tag)
 
         # `d` is a testing-only convenience. We must wait for it to fire, and
         # we must do that from the reactor thread.

=== modified file 'src/maasserver/triggers/system.py'
--- src/maasserver/triggers/system.py	2016-10-04 12:20:56 +0000
+++ src/maasserver/triggers/system.py	2016-10-12 15:26:17 +0000
@@ -924,13 +924,14 @@
 
 # Triggered when a config is inserted. Increments the zone serial and notifies
 # that DNS needs to be updated. Only watches for inserts on config
-# upstream_dns, default_dns_ttl, and windows_kms_host.
+# upstream_dns, dnssec_validation, default_dns_ttl, and windows_kms_host.
 DNS_CONFIG_INSERT = dedent("""\
     CREATE OR REPLACE FUNCTION sys_dns_config_insert()
     RETURNS trigger as $$
     BEGIN
       -- Only care about the
       IF (NEW.name = 'upstream_dns' OR
+          NEW.name = 'dnssec_validation' OR
           NEW.name = 'default_dns_ttl' OR
           NEW.name = 'windows_kms_host')
       THEN
@@ -948,7 +949,7 @@
 
 # Triggered when a config is updated. Increments the zone serial and notifies
 # that DNS needs to be updated. Only watches for updates on config
-# upstream_dns, default_dns_ttl, and windows_kms_host.
+# upstream_dns, dnssec_validation, default_dns_ttl, and windows_kms_host.
 DNS_CONFIG_UPDATE = dedent("""\
     CREATE OR REPLACE FUNCTION sys_dns_config_update()
     RETURNS trigger as $$
@@ -957,6 +958,7 @@
       -- windows_kms_host.
       IF (OLD.value != NEW.value AND (
           NEW.name = 'upstream_dns' OR
+          NEW.name = 'dnssec_validation' OR
           NEW.name = 'default_dns_ttl' OR
           NEW.name = 'windows_kms_host'))
       THEN

=== modified file 'src/maasserver/triggers/tests/test_system_listener.py'
--- src/maasserver/triggers/tests/test_system_listener.py	2016-10-04 12:20:56 +0000
+++ src/maasserver/triggers/tests/test_system_listener.py	2016-10-12 15:26:17 +0000
@@ -3194,6 +3194,29 @@
 
     @wait_for_reactor
     @inlineCallbacks
+    def test_sends_message_for_config_dnssec_validation_insert(self):
+        yield deferToDatabase(register_system_triggers)
+        yield self.capturePublication()
+        dv = DeferredValue()
+        listener = self.make_listener_without_delay()
+        listener.register(
+            "sys_dns", lambda *args: dv.set(args))
+        yield listener.startService()
+        try:
+            yield deferToDatabase(
+                Config.objects.set_config,
+                "dnssec_validation", "no")
+            yield dv.get(timeout=2)
+            yield self.assertPublicationUpdated()
+        finally:
+            yield listener.stopService()
+        self.assertThat(
+            self.getCapturedPublication().source, Equals(
+                "Configuration dnssec_validation set to %s"
+                % json.dumps("no")))
+
+    @wait_for_reactor
+    @inlineCallbacks
     def test_sends_message_for_config_default_dns_ttl_insert(self):
         default_dns_ttl_new = random.randint(10, 1000)
         yield deferToDatabase(register_system_triggers)
@@ -3271,6 +3294,33 @@
 
     @wait_for_reactor
     @inlineCallbacks
+    def test_sends_message_for_config_dnssec_validation_update(self):
+        yield deferToDatabase(register_system_triggers)
+        yield deferToDatabase(
+            Config.objects.set_config,
+            "dnssec_validation", "no")
+        yield self.capturePublication()
+        dv = DeferredValue()
+        listener = self.make_listener_without_delay()
+        listener.register(
+            "sys_dns", lambda *args: dv.set(args))
+        yield listener.startService()
+        try:
+            yield deferToDatabase(
+                Config.objects.set_config,
+                "dnssec_validation", "yes")
+            yield dv.get(timeout=2)
+            yield self.assertPublicationUpdated()
+        finally:
+            yield listener.stopService()
+        self.assertThat(
+            self.getCapturedPublication().source, Equals(
+                "Configuration dnssec_validation changed from %s to %s"
+                % (json.dumps("no"),
+                   json.dumps("yes"))))
+
+    @wait_for_reactor
+    @inlineCallbacks
     def test_sends_message_for_config_default_dns_ttl_update(self):
         default_dns_ttl_old = random.randint(10, 1000)
         default_dns_ttl_new = random.randint(10, 1000)

=== modified file 'src/maasserver/triggers/tests/test_websocket.py'
--- src/maasserver/triggers/tests/test_websocket.py	2016-09-22 02:53:33 +0000
+++ src/maasserver/triggers/tests/test_websocket.py	2016-10-12 15:26:17 +0000
@@ -99,6 +99,9 @@
             "dhcpsnippet_dhcpsnippet_create_notify",
             "dhcpsnippet_dhcpsnippet_update_notify",
             "dhcpsnippet_dhcpsnippet_delete_notify",
+            "iprange_iprange_subnet_insert_notify",
+            "iprange_iprange_subnet_update_notify",
+            "iprange_iprange_subnet_delete_notify",
             ]
         sql, args = psql_array(triggers, sql_type="text")
         with closing(connection.cursor()) as cursor:

=== modified file 'src/maasserver/triggers/tests/test_websocket_listener.py'
--- src/maasserver/triggers/tests/test_websocket_listener.py	2016-09-22 02:53:33 +0000
+++ src/maasserver/triggers/tests/test_websocket_listener.py	2016-10-12 15:26:17 +0000
@@ -13,6 +13,7 @@
 from crochet import wait_for
 from maasserver.enum import (
     IPADDRESS_TYPE,
+    IPRANGE_TYPE,
     NODE_TYPE,
 )
 from maasserver.listener import PostgresListenerService
@@ -2980,6 +2981,127 @@
             yield listener.stopService()
 
 
+class TestIPRangeSubnetListener(
+        MAASTransactionServerTestCase, TransactionalHelpersMixin):
+    """End-to-end test of both the listeners code and the triggers on
+    maasserver_iprange tables that notifies affected subnets."""
+
+    @wait_for_reactor
+    @inlineCallbacks
+    def test__calls_handler_on_create_notification(self):
+        yield deferToDatabase(register_websocket_triggers)
+        subnet = yield deferToDatabase(
+            self.create_subnet, {
+                "cidr": '192.168.0.0/24',
+                "gateway_ip": '192.168.0.1',
+                "dns_servers": [],
+                })
+
+        listener = PostgresListenerService()
+        dv = DeferredValue()
+        listener.register("subnet", lambda *args: dv.set(args))
+        yield listener.startService()
+        try:
+            iprange = yield deferToDatabase(
+                self.create_iprange, {
+                    "type": IPRANGE_TYPE.DYNAMIC,
+                    "subnet": subnet,
+                    "start_ip": '192.168.0.100',
+                    "end_ip": '192.168.0.110',
+                    })
+            yield dv.get(timeout=2)
+            self.assertEqual(('update', '%s' % iprange.subnet.id), dv.value)
+        finally:
+            yield listener.stopService()
+
+    @wait_for_reactor
+    @inlineCallbacks
+    def test__calls_handler_on_update_notification(self):
+        yield deferToDatabase(register_websocket_triggers)
+        iprange = yield deferToDatabase(self.create_iprange)
+        new_end_ip = factory.pick_ip_in_IPRange(iprange)
+
+        listener = PostgresListenerService()
+        dv = DeferredValue()
+        listener.register("subnet", lambda *args: dv.set(args))
+        yield listener.startService()
+        try:
+            yield deferToDatabase(
+                self.update_iprange,
+                iprange.id, {"end_ip": new_end_ip})
+            yield dv.get(timeout=2)
+            self.assertEqual(('update', '%s' % iprange.subnet.id), dv.value)
+        finally:
+            yield listener.stopService()
+
+    @wait_for_reactor
+    @inlineCallbacks
+    def test__calls_handler_on_update_on_old_and_new_subnet_notification(self):
+        yield deferToDatabase(register_websocket_triggers)
+        old_subnet = yield deferToDatabase(
+            self.create_subnet, {
+                "cidr": '192.168.0.0/24',
+                "gateway_ip": '192.168.0.1',
+                "dns_servers": [],
+                })
+        new_subnet = yield deferToDatabase(
+            self.create_subnet, {
+                "cidr": '192.168.1.0/24',
+                "gateway_ip": '192.168.1.1',
+                "dns_servers": [],
+                })
+        iprange = yield deferToDatabase(
+            self.create_iprange, {
+                "type": IPRANGE_TYPE.DYNAMIC,
+                "subnet": old_subnet,
+                "start_ip": '192.168.0.100',
+                "end_ip": '192.168.0.110',
+                })
+        dvs = [DeferredValue(), DeferredValue()]
+
+        def set_defer_value(*args):
+            for dv in dvs:
+                if not dv.isSet:
+                    dv.set(args)
+                    break
+
+        listener = PostgresListenerService()
+        listener.register("subnet", set_defer_value)
+        yield listener.startService()
+        try:
+            yield deferToDatabase(self.update_iprange, iprange.id, {
+                "type": IPRANGE_TYPE.DYNAMIC,
+                "subnet": new_subnet,
+                "start_ip": '192.168.1.10',
+                "end_ip": '192.168.1.150',
+                })
+            yield dvs[0].get(timeout=2)
+            yield dvs[1].get(timeout=2)
+            self.assertItemsEqual([
+                ('update', '%s' % old_subnet.id),
+                ('update', '%s' % new_subnet.id),
+                ], [dvs[0].value, dvs[1].value])
+        finally:
+            yield listener.stopService()
+
+    @wait_for_reactor
+    @inlineCallbacks
+    def test__calls_handler_on_delete_notification(self):
+        yield deferToDatabase(register_websocket_triggers)
+        iprange = yield deferToDatabase(self.create_iprange)
+
+        listener = PostgresListenerService()
+        dv = DeferredValue()
+        listener.register("subnet", lambda *args: dv.set(args))
+        yield listener.startService()
+        try:
+            yield deferToDatabase(self.delete_iprange, iprange.id)
+            yield dv.get(timeout=2)
+            self.assertEqual(('update', '%s' % iprange.subnet.id), dv.value)
+        finally:
+            yield listener.stopService()
+
+
 class TestNodeTypeChange(
         MAASTransactionServerTestCase, TransactionalHelpersMixin):
     """End-to-end test of node type change triggers code."""

=== modified file 'src/maasserver/triggers/websocket.py'
--- src/maasserver/triggers/websocket.py	2016-09-22 02:53:33 +0000
+++ src/maasserver/triggers/websocket.py	2016-10-12 15:26:17 +0000
@@ -687,6 +687,50 @@
     $$ LANGUAGE plpgsql;
     """)
 
+# Procedure that is called when an IP range is created to update its related
+# subnet.
+IP_RANGE_SUBNET_INSERT_NOTIFY = dedent("""\
+    CREATE OR REPLACE FUNCTION %s() RETURNS trigger AS $$
+    BEGIN
+      IF NEW.subnet_id IS NOT NULL THEN
+        PERFORM pg_notify('subnet_update',CAST(NEW.subnet_id AS text));
+      END IF;
+      RETURN NEW;
+    END;
+    $$ LANGUAGE plpgsql;
+    """)
+
+# Procedure that is called when an IP range is updated to update its related
+# subnet.
+IP_RANGE_SUBNET_UPDATE_NOTIFY = dedent("""\
+    CREATE OR REPLACE FUNCTION %s() RETURNS trigger AS $$
+    BEGIN
+      IF OLD.subnet_id != NEW.subnet_id THEN
+        IF OLD.subnet_id IS NOT NULL THEN
+          PERFORM pg_notify('subnet_update',CAST(OLD.subnet_id AS text));
+        END IF;
+      END IF;
+      IF NEW.subnet_id IS NOT NULL THEN
+        PERFORM pg_notify('subnet_update',CAST(NEW.subnet_id AS text));
+      END IF;
+      RETURN NEW;
+    END;
+    $$ LANGUAGE plpgsql;
+    """)
+
+# Procedure that is called when an IP range is deleted to update its related
+# subnet.
+IP_RANGE_SUBNET_DELETE_NOTIFY = dedent("""\
+    CREATE OR REPLACE FUNCTION %s() RETURNS trigger AS $$
+    BEGIN
+      IF OLD.subnet_id IS NOT NULL THEN
+        PERFORM pg_notify('subnet_update',CAST(OLD.subnet_id AS text));
+      END IF;
+      RETURN OLD;
+    END;
+    $$ LANGUAGE plpgsql;
+    """)
+
 # Procedure that is called when a DNSData entry is changed.
 DNSDATA_DOMAIN_NOTIFY = dedent("""\
     CREATE OR REPLACE FUNCTION %s() RETURNS trigger AS $$
@@ -1134,6 +1178,23 @@
         "maasserver_staticipaddress",
         "ipaddress_domain_delete_notify", "delete")
 
+    # IP range subnet notifications
+    register_procedure(
+        IP_RANGE_SUBNET_INSERT_NOTIFY % 'iprange_subnet_insert_notify')
+    register_procedure(
+        IP_RANGE_SUBNET_UPDATE_NOTIFY % 'iprange_subnet_update_notify')
+    register_procedure(
+        IP_RANGE_SUBNET_DELETE_NOTIFY % 'iprange_subnet_delete_notify')
+    register_trigger(
+        "maasserver_iprange",
+        "iprange_subnet_insert_notify", "insert")
+    register_trigger(
+        "maasserver_iprange",
+        "iprange_subnet_update_notify", "update")
+    register_trigger(
+        "maasserver_iprange",
+        "iprange_subnet_delete_notify", "delete")
+
     # DNSData table
     register_procedure(
         DNSDATA_DOMAIN_NOTIFY % (

=== modified file 'src/maasserver/utils/threads.py'
--- src/maasserver/utils/threads.py	2015-12-01 18:12:59 +0000
+++ src/maasserver/utils/threads.py	2016-10-12 15:26:17 +0000
@@ -1,4 +1,4 @@
-# Copyright 2015 Canonical Ltd.  This software is licensed under the
+# Copyright 2015-2016 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Stuff relating to threads in the MAAS Region Controller.
@@ -30,9 +30,11 @@
     ThreadPool,
     ThreadUnpool,
 )
-from twisted.internet import reactor
+from twisted.internet import (
+    reactor,
+    threads,
+)
 from twisted.internet.defer import DeferredSemaphore
-from twisted.internet.threads import deferToThreadPool
 
 
 max_threads_for_default_pool = 50
@@ -135,7 +137,7 @@
 
 def deferToDatabase(func, *args, **kwargs):
     """Call `func` in a thread where database activity is permitted."""
-    return deferToThreadPool(
+    return threads.deferToThreadPool(
         reactor, reactor.threadpoolForDatabase,
         func, *args, **kwargs)
 

=== modified file 'src/provisioningserver/import_images/download_resources.py'
--- src/provisioningserver/import_images/download_resources.py	2016-09-30 13:56:04 +0000
+++ src/provisioningserver/import_images/download_resources.py	2016-10-12 15:26:17 +0000
@@ -285,6 +285,10 @@
 
         osystem = get_os_from_product(item)
         subarches = self.product_mapping.get(item)
+        if item.get('rolling', False):
+            subarch_parts = item['subarch'].split('-')
+            subarch_parts[1] = 'rolling'
+            subarches.append('-'.join(subarch_parts))
         link_resources(
             snapshot_path=self.root_path, links=links,
             osystem=osystem, arch=item['arch'], release=item['release'],

=== modified file 'src/provisioningserver/import_images/tests/test_download_resources.py'
--- src/provisioningserver/import_images/tests/test_download_resources.py	2016-09-08 18:38:49 +0000
+++ src/provisioningserver/import_images/tests/test_download_resources.py	2016-10-12 15:26:17 +0000
@@ -186,6 +186,9 @@
         if ftype is None:
             ftype = factory.make_name('ftype')
         return {
+            'content_id': 'maas:v2:download',
+            'product_name': factory.make_string(),
+            'version_name': datetime.utcnow().strftime('%Y%m%d'),
             'sha256': factory.make_name('sha256'),
             'size': random.randint(2, 2**16),
             'ftype': ftype,
@@ -198,11 +201,13 @@
 
     def test_inserts_archive(self):
         product = self.make_product('archive.tar.xz')
-        # Mock in test data
-        repo_writer = download_resources.RepoWriter(None, None, None)
+        product_mapping = ProductMapping()
+        subarch = factory.make_name('subarch')
+        product_mapping.add(product, subarch)
+        repo_writer = download_resources.RepoWriter(
+            None, None, product_mapping)
         self.patch(
             download_resources, 'products_exdata').return_value = product
-        self.patch(repo_writer, 'product_mapping')
         mock_extract_archive_tar = self.patch(
             download_resources, 'extract_archive_tar')
         mock_link_resources = self.patch(download_resources, 'link_resources')
@@ -217,16 +222,18 @@
             MockCalledOnceWith(
                 snapshot_path=None, links=mock.ANY, osystem=product['os'],
                 arch=product['arch'], release=product['release'],
-                label=product['label'], subarches=mock.ANY,
+                label=product['label'], subarches=[subarch],
                 bootloader_type=None))
 
     def test_inserts_root_image(self):
         product = self.make_product('root-image.gz')
-        # Mock in test data
-        repo_writer = download_resources.RepoWriter(None, None, None)
+        product_mapping = ProductMapping()
+        subarch = factory.make_name('subarch')
+        product_mapping.add(product, subarch)
+        repo_writer = download_resources.RepoWriter(
+            None, None, product_mapping)
         self.patch(
             download_resources, 'products_exdata').return_value = product
-        self.patch(repo_writer, 'product_mapping')
         mock_insert_root_image = self.patch(
             download_resources, 'insert_root_image')
         mock_link_resources = self.patch(download_resources, 'link_resources')
@@ -241,30 +248,58 @@
             MockCalledOnceWith(
                 snapshot_path=None, links=mock.ANY, osystem=product['os'],
                 arch=product['arch'], release=product['release'],
-                label=product['label'], subarches=mock.ANY,
+                label=product['label'], subarches=[subarch],
                 bootloader_type=None))
 
     def test_inserts_file(self):
         product = self.make_product()
-        # Mock in test data
-        repo_writer = download_resources.RepoWriter(None, None, None)
-        self.patch(
-            download_resources, 'products_exdata').return_value = product
-        self.patch(repo_writer, 'product_mapping')
-        mock_insert_file = self.patch(download_resources, 'insert_file')
-        mock_link_resources = self.patch(download_resources, 'link_resources')
-        repo_writer.insert_item(product, None, None, None, None)
-        self.assertThat(
-            mock_insert_file,
-            MockCalledOnceWith(
-                mock.ANY, os.path.basename(product['path']), product['sha256'],
-                {'sha256': product['sha256']}, product['size'], None))
-        self.assertThat(
-            mock_link_resources,
-            MockCalledOnceWith(
-                snapshot_path=None, links=mock.ANY, osystem=product['os'],
-                arch=product['arch'], release=product['release'],
-                label=product['label'], subarches=mock.ANY,
+        product_mapping = ProductMapping()
+        subarch = factory.make_name('subarch')
+        product_mapping.add(product, subarch)
+        repo_writer = download_resources.RepoWriter(
+            None, None, product_mapping)
+        self.patch(
+            download_resources, 'products_exdata').return_value = product
+        mock_insert_file = self.patch(download_resources, 'insert_file')
+        mock_link_resources = self.patch(download_resources, 'link_resources')
+        repo_writer.insert_item(product, None, None, None, None)
+        self.assertThat(
+            mock_insert_file,
+            MockCalledOnceWith(
+                mock.ANY, os.path.basename(product['path']), product['sha256'],
+                {'sha256': product['sha256']}, product['size'], None))
+        self.assertThat(
+            mock_link_resources,
+            MockCalledOnceWith(
+                snapshot_path=None, links=mock.ANY, osystem=product['os'],
+                arch=product['arch'], release=product['release'],
+                label=product['label'], subarches=[subarch],
+                bootloader_type=None))
+
+    def test_inserts_rolling_links(self):
+        product = self.make_product()
+        product['subarch'] = 'hwe-16.04'
+        product['rolling'] = True
+        product_mapping = ProductMapping()
+        product_mapping.add(product, 'hwe-16.04')
+        repo_writer = download_resources.RepoWriter(
+            None, None, product_mapping)
+        self.patch(
+            download_resources, 'products_exdata').return_value = product
+        mock_insert_file = self.patch(download_resources, 'insert_file')
+        mock_link_resources = self.patch(download_resources, 'link_resources')
+        repo_writer.insert_item(product, None, None, None, None)
+        self.assertThat(
+            mock_insert_file,
+            MockCalledOnceWith(
+                mock.ANY, os.path.basename(product['path']), product['sha256'],
+                {'sha256': product['sha256']}, product['size'], None))
+        self.assertThat(
+            mock_link_resources,
+            MockCalledOnceWith(
+                snapshot_path=None, links=mock.ANY, osystem=product['os'],
+                arch=product['arch'], release=product['release'],
+                label=product['label'], subarches=['hwe-16.04', 'hwe-rolling'],
                 bootloader_type=None))
 
 

=== modified file 'src/provisioningserver/monkey.py'
--- src/provisioningserver/monkey.py	2016-08-18 16:29:16 +0000
+++ src/provisioningserver/monkey.py	2016-10-12 15:26:17 +0000
@@ -312,6 +312,21 @@
     twisted.internet.tcp._resolveIPv6 = new_resolveIPv6
 
 
+def augment_twisted_deferToThreadPool():
+    """Wrap every function deferred to a thread in `synchronous`."""
+    from twisted.internet import threads
+    from twisted.internet.threads import deferToThreadPool
+    from provisioningserver.utils.twisted import ISynchronous, synchronous
+
+    def new_deferToThreadPool(reactor, threadpool, f, *args, **kwargs):
+        """Variant of Twisted's that wraps all functions in `synchronous`."""
+        func = f if ISynchronous.providedBy(f) else synchronous(f)
+        return deferToThreadPool(reactor, threadpool, func, *args, **kwargs)
+
+    if threads.deferToThreadPool.__module__ != __name__:
+        threads.deferToThreadPool = new_deferToThreadPool
+
+
 def add_patches_to_txtftp():
     add_term_error_code_to_tftp()
     fix_tftp_requests()
@@ -322,3 +337,4 @@
     fix_twisted_web_http_Request()
     fix_twisted_web_server_addressToTuple()
     fix_twisted_internet_tcp()
+    augment_twisted_deferToThreadPool()

=== modified file 'src/provisioningserver/tests/test_monkey.py'
--- src/provisioningserver/tests/test_monkey.py	2016-08-18 16:29:16 +0000
+++ src/provisioningserver/tests/test_monkey.py	2016-10-12 15:26:17 +0000
@@ -3,27 +3,25 @@
 
 """Test monkey patches."""
 
-__all__ = [
-    ]
-
-
-import sys
+__all__ = []
+
 from unittest.mock import sentinel
 
-from maastesting.testcase import MAASTestCase
-from provisioningserver.monkey import add_patches_to_txtftp
+from maastesting.testcase import (
+    MAASTestCase,
+    MAASTwistedRunTest,
+)
+from provisioningserver.monkey import (
+    add_patches_to_txtftp,
+    augment_twisted_deferToThreadPool,
+)
+from testtools.deferredruntest import assert_fails_with
 import tftp.datagram
-
-
-if sys.version_info > (3, 0):
-    import urllib.request as urllib_request
-    import urllib.error as urllib_error
-else:
-    import urllib2 as urllib_request
-    urllib_error = urllib_request
-
-
-class TestAddTermErrorCodeToTFT(MAASTestCase):
+from twisted.internet.defer import Deferred
+from twisted.internet.threads import deferToThread
+
+
+class TestAddTermErrorCodeToTFTP(MAASTestCase):
 
     def test_adds_error_code_8(self):
         self.patch(tftp.datagram, 'errors', {})
@@ -38,3 +36,19 @@
         add_patches_to_txtftp()
         self.assertEqual(
             sentinel.error_8, tftp.datagram.errors.get(8))
+
+
+class TestAugmentDeferToThreadPool(MAASTestCase):
+    """Tests for `augment_twisted_deferToThreadPool`."""
+
+    run_tests_with = MAASTwistedRunTest.make_factory(timeout=5)
+
+    def setUp(self):
+        super(TestAugmentDeferToThreadPool, self).setUp()
+        augment_twisted_deferToThreadPool()
+
+    def test_functions_returning_Deferreds_from_threads_crash(self):
+        return assert_fails_with(deferToThread(Deferred), TypeError)
+
+    def test_functions_returning_other_from_threads_are_okay(self):
+        return deferToThread(round, 12.34).addCallback(self.assertEqual, 12)

=== modified file 'src/provisioningserver/utils/tests/test_twisted.py'
--- src/provisioningserver/utils/tests/test_twisted.py	2016-09-26 06:36:07 +0000
+++ src/provisioningserver/utils/tests/test_twisted.py	2016-10-12 15:26:17 +0000
@@ -273,6 +273,33 @@
         self.assertThat(synchronous(return_args), Provides(ISynchronous))
 
 
+class TestSynchronousDecoratorSychronously(MAASTestCase):
+    """Test `synchronous` outside of the reactor thread."""
+
+    def test__raises_TypeError_when_call_returns_Deferred(self):
+        @synchronous
+        def deferSomething(*args, **kwargs):
+            return Deferred()
+
+        a, b = factory.make_name("a"), factory.make_name("b")
+        error = self.assertRaises(TypeError, deferSomething, a, b=b)
+        self.assertThat(str(error), Equals(
+            "Synchronous call returned a Deferred: %s(%r, b=%r)"
+            % (deferSomething.__qualname__, a, b)))
+
+    def test__raises_TypeError_when_callable_returns_Deferred(self):
+        class Something:
+            def __call__(self, *args, **kwargs):
+                return Deferred()
+
+        something = Something()
+        a, b = factory.make_name("a"), factory.make_name("b")
+        error = self.assertRaises(TypeError, synchronous(something), a, b=b)
+        self.assertThat(str(error), Equals(
+            "Synchronous call returned a Deferred: %s(%r, b=%r)"
+            % (Something.__qualname__, a, b)))
+
+
 class TestRetries(MAASTestCase):
 
     def assertRetry(

=== modified file 'src/provisioningserver/utils/twisted.py'
--- src/provisioningserver/utils/twisted.py	2016-09-27 13:19:40 +0000
+++ src/provisioningserver/utils/twisted.py	2016-10-12 15:26:17 +0000
@@ -36,7 +36,11 @@
     partial,
     wraps,
 )
-from itertools import repeat
+from itertools import (
+    chain,
+    repeat,
+    starmap,
+)
 from operator import attrgetter
 import os
 from os import (
@@ -198,6 +202,13 @@
 
     :raises AssertionError: When called inside the reactor thread.
     """
+    try:
+        # A function or method; see PEP 3155.
+        func_name = func.__qualname__
+    except AttributeError:
+        # An instance with a __call__ method.
+        func_name = type(func).__qualname__
+
     @wraps(func)
     def wrapper(*args, **kwargs):
         # isInIOThread() can return True if the reactor has previously been
@@ -208,7 +219,16 @@
                 "Function %s(...) must not be called in the "
                 "reactor thread." % func.__name__)
         else:
-            return func(*args, **kwargs)
+            result = func(*args, **kwargs)
+            if isinstance(result, Deferred):
+                args_reprs = chain(
+                    map(repr, args), starmap(
+                        "{}={!r}".format, kwargs.items()))
+                raise TypeError(
+                    "Synchronous call returned a Deferred: %s(%s)"
+                    % (func_name, ", ".join(args_reprs)))
+            else:
+                return result
 
     # This makes it possible to reliably determine programmatically if a
     # function has been decorated with @synchronous.

=== modified file 'versions.cfg'
--- versions.cfg	2016-05-25 15:38:40 +0000
+++ versions.cfg	2016-10-12 15:26:17 +0000
@@ -11,38 +11,45 @@
 blessings = 1.6
 buildout-versions = 1.7
 coverage = 4.1
+decorator = 4.0.10
 django-nose = 1.4.2
 extras = 0.0.3
 fixtures = 2.0.0
 hypothesis = 1.11.4
-ipdb = 0.8.1
-ipython = 1.2.1
+ipdb = 0.10.1
+ipython = 5.1.0
+ipython-genutils = 0.1.0
 junitxml = 0.6
 linecache2 = 1.0.0
 nose = 1.3.1
 nose-progressive = 1.5.1
 nose-timer = 0.5.0
 pbr = 1.8.1
+pickleshare = 0.7.4
 postgresfixture = 0.3.1
+prompt-toolkit = 1.0.7
 python-mimeparse = 0.1.4
 python-subunit = 1.2.0
 selenium = 2.45
 setuptools = 18.7
+simplegeneric = 0.8.1
 sqlparse = 0.1.10
 termcolor = 1.1.0
 testresources = 0.2.7
 testscenarios = 0.4
 testtools = 2.1.0
 traceback2 = 1.4.0
+traitlets = 4.3.1
 unittest2 = 1.1.0
+wcwidth = 0.1.7
 zc.buildout = 2.5.0
 zc.recipe.egg = 2.0.3
 
 # Lint.
-flake8 = 2.5.4
-mccabe = 0.2.1
-pep8 = 1.7.0
-pyflakes = 0.8.1
+flake8 = 3.0.4
+mccabe = 0.5.2
+pycodestyle = 2.0.0
+pyflakes = 1.1.0
 
 # Explicit versions for PACKAGED dependencies. This is where a TEST
 # dependency itself depends on an INSTALL/RUNTIME dependency which is,

